@@ -25,7 +25,6 @@ import (
 	"github.com/ubuntu/decorate"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/credentials"
-	"google.golang.org/grpc/credentials/insecure"
 )
 
 // Daemon is a grpc daemon with systemd support.
@@ -241,6 +240,9 @@ func (d *Daemon) systemdNotifyReady(ctx context.Context) error {
 	}
 	if sent {
 		log.Debug(ctx, i18n.G("Ready state sent to systemd"))
+		if err := os.Unsetenv("NOTIFY_SOCKET"); err != nil {
+			log.Warningf(ctx, "couldn't unset NOTIFY_SOCKET for subprocesses: %v", err)
+		}
 	}
 	return nil
 }
@@ -291,21 +293,14 @@ func (d *Daemon) connect(ctx context.Context) (server *streams.Server, err error
 	if err != nil {
 		return nil, err
 	}
-	conn, err := grpc.DialContext(ctx, addr,
-		grpc.WithTransportCredentials(insecure.NewCredentials()),
+	conn, err := grpc.NewClient(addr,
 		grpc.WithStreamInterceptor(interceptorschain.StreamClient(
 			log.StreamClientInterceptor(logrus.StandardLogger(), log.WithClientID(distroName)),
 		)), grpc.WithTransportCredentials(credentials.NewTLS(tlsConfig)))
 	if err != nil {
-		return nil, fmt.Errorf("could not dial: %v", err)
+		return nil, fmt.Errorf("could not create a gRPC client: %v", err)
 	}
 
-	defer func(err *error) {
-		if *err != nil {
-			conn.Close()
-		}
-	}(&err)
-
 	return streams.NewServer(ctx, d.system, conn), nil
 }
 
