@@ -3,6 +3,7 @@ package system
 import (
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -18,24 +19,8 @@ const (
 )
 
 // LandscapeEnable registers the current distro to Landscape with the specified config.
-func (s *System) LandscapeEnable(ctx context.Context, landscapeConfig string, hostagentUID string) (err error) {
-	// Decorating here to avoid stuttering the URL (url package prints it as well)
-	defer decorate.OnError(&err, "could not register distro to Landscape")
-
-	if landscapeConfig, err = modifyConfig(ctx, s, landscapeConfig, hostagentUID); err != nil {
-		return err
-	}
-
-	if err := s.writeConfig(landscapeConfig); err != nil {
-		return err
-	}
-
-	cmd := s.backend.LandscapeConfigExecutable(ctx, "--config", landscapeConfigPath, "--silent")
-	if _, err := runCommand(cmd); err != nil {
-		return fmt.Errorf("could not enable Landscape: %v", err)
-	}
-
-	return nil
+func (s *System) LandscapeEnable(ctx context.Context, landscapeConfig string) (err error) {
+	return s.fixAndEnableLandscapeFromConfig(ctx, landscapeConfig, true)
 }
 
 // LandscapeDisable unregisters the current distro from Landscape.
@@ -48,6 +33,56 @@ func (s *System) LandscapeDisable(ctx context.Context) (err error) {
 	return nil
 }
 
+// EnsureValidLandscapeConfig ensures that the Landscape configuration is valid and enabled.
+func (s *System) EnsureValidLandscapeConfig(ctx context.Context) (err error) {
+	defer decorate.OnError(&err, "could not ensure valid Landscape configuration")
+
+	landscapeConfig, err := os.ReadFile(s.Path(landscapeConfigPath))
+	if err != nil {
+		if errors.Is(err, os.ErrNotExist) {
+			log.Debug(ctx, "No Landscape configuration detected: nothing to do")
+			return nil
+		}
+		return err
+	}
+
+	return s.fixAndEnableLandscapeFromConfig(ctx, string(landscapeConfig), false)
+}
+
+func (s *System) fixAndEnableLandscapeFromConfig(ctx context.Context, landscapeConfig string, enableUnconditionally bool) (err error) {
+	// Decorating here to avoid stuttering the URL (url package prints it as well)
+	defer decorate.OnError(&err, "could not register distro to Landscape")
+
+	r := strings.NewReader(landscapeConfig)
+	iniFile, err := ini.Load(r)
+	if err != nil {
+		return fmt.Errorf("could not parse config: %v", err)
+	}
+
+	modifiedLandscapeConfig, err := normalizeLandscapeConfig(ctx, s, iniFile)
+	if err != nil {
+		return err
+	}
+
+	// No change to do, do not rewrite config
+	if !enableUnconditionally && modifiedLandscapeConfig == landscapeConfig {
+		log.Debug(ctx, "Landscape configuration is already valid")
+		return nil
+	}
+
+	if err := s.writeConfig(modifiedLandscapeConfig); err != nil {
+		return err
+	}
+
+	// TODO: check foreground/background
+	cmd := s.backend.LandscapeConfigExecutable(ctx, "--config", landscapeConfigPath, "--silent")
+	if _, err := runCommand(cmd); err != nil {
+		return fmt.Errorf("could not enable Landscape: %v", err)
+	}
+
+	return nil
+}
+
 func (s *System) writeConfig(landscapeConfig string) (err error) {
 	defer decorate.OnError(&err, "could not write Landscape configuration")
 
@@ -86,94 +121,60 @@ func (s *System) writeConfig(landscapeConfig string) (err error) {
 	return nil
 }
 
-// modifyConfig overrides parameters in the configuration to adapt them to the current distro.
-func modifyConfig(ctx context.Context, s *System, landscapeConfig string, hostagentUID string) (string, error) {
-	if landscapeConfig == "" {
-		return "", nil
-	}
-
-	r := strings.NewReader(landscapeConfig)
-	data, err := ini.Load(r)
+// normalizeLandscapeConfig ensures that the landscape config has the expected computer_title and SSL certificate path
+// transformed in a Linux path.
+func normalizeLandscapeConfig(ctx context.Context, s *System, iniFile *ini.File) (modifiedLandscapeConfig string, err error) {
+	clientSection, err := iniFile.GetSection("client")
 	if err != nil {
-		return "", fmt.Errorf("could not parse config: %v", err)
+		return "", err
 	}
 
-	data.DeleteSection("host")
-
+	// Add or refresh computer title
 	distroName, err := s.WslDistroName(ctx)
 	if err != nil {
 		return "", err
 	}
-	if err := createKey(ctx, data, "client", "computer_title", distroName, true); err != nil {
-		return "", err
+	oldComputerTitle, err := clientSection.GetKey("computer_title")
+	if err != nil {
+		if _, err = clientSection.NewKey("computer_title", distroName); err != nil {
+			return "", err
+		}
+	} else if oldComputerTitle.String() != distroName {
+		oldComputerTitle.SetValue(distroName)
 	}
 
-	if err := createKey(ctx, data, "client", "hostagent_uid", hostagentUID, true); err != nil {
-		return "", err
-	}
-
-	if err := createKey(ctx, data, "client", "tags", "wsl", false); err != nil {
-		return "", err
-	}
-
-	if err := overrideSSLCertificate(ctx, s, data); err != nil {
+	// Refresh SSL certificate path if any
+	if err := overrideSSLCertificate(ctx, s, clientSection); err != nil {
 		return "", fmt.Errorf("could not override SSL certificate path: %v", err)
 	}
 
+	// Return the modified config as a string.
 	w := &bytes.Buffer{}
-	if _, err := data.WriteTo(w); err != nil {
-		return "", fmt.Errorf("could not write modified config: %v", err)
+	if _, err := iniFile.WriteTo(w); err != nil {
+		return "", fmt.Errorf("could not regenerate modified config: %v", err)
 	}
 
 	return w.String(), nil
 }
 
-// createKey tries to create a key with a particular value, optionally overriding an existing key.
-func createKey(ctx context.Context, data *ini.File, section, key, value string, override bool) error {
-	sec, err := data.GetSection(section)
-	if err != nil {
-		if sec, err = data.NewSection(section); err != nil {
-			return fmt.Errorf("could not find nor create section %q: %v", section, err)
-		}
-	}
-
-	if sec.HasKey(key) {
-		if !override {
-			log.Infof(ctx, "Landscape config contains key %q. Its value will not be overridden.", key)
-			return nil
-		}
-
-		log.Infof(ctx, "Landscape config contains key %q. Its value will be overridden.", key)
-		sec.DeleteKey(key)
-	}
-
-	if _, err := sec.NewKey(key, value); err != nil {
-		return fmt.Errorf("could not create key %q: %v", key, err)
-	}
-
-	return nil
-}
-
 // overrideComputerTitle converts the ssl_public_key field in the Landscape config
 // from a Windows path to a Linux path.
-func overrideSSLCertificate(ctx context.Context, s *System, data *ini.File) error {
-	const section = "client"
+func overrideSSLCertificate(ctx context.Context, s *System, section *ini.Section) error {
 	const key = "ssl_public_key"
 
-	sec, err := data.GetSection(section)
+	k, err := section.GetKey(key)
 	if err != nil {
-		// No certificate
-		return nil
-	}
-
-	k, err := sec.GetKey(key)
-	if err != nil {
-		// No certificate
+		// No certificate: nothing to transform
 		return nil
 	}
 
 	pathWindows := k.String()
 
+	if len(pathWindows) == 0 {
+		// Empty paths are translated by wslpath as the current working directory, which is not what we want.
+		return nil
+	}
+
 	cmd := s.backend.WslpathExecutable(ctx, "-ua", pathWindows)
 	out, err := runCommand(cmd)
 	if err != nil {
@@ -182,6 +183,5 @@ func overrideSSLCertificate(ctx context.Context, s *System, data *ini.File) erro
 
 	pathLinux := s.Path(strings.TrimSpace(string(out)))
 	k.SetValue(pathLinux)
-
 	return nil
 }
