@@ -15,6 +15,7 @@ import (
 	"strings"
 
 	agentapi "github.com/canonical/ubuntu-pro-for-wsl/agentapi/go"
+	log "github.com/canonical/ubuntu-pro-for-wsl/common/grpc/logstreamer"
 	"github.com/ubuntu/decorate"
 	"gopkg.in/ini.v1"
 )
@@ -70,9 +71,15 @@ func New(args ...Option) *System {
 		f(&opts)
 	}
 
-	return &System{
+	s := &System{
 		backend: opts.backend,
 	}
+
+	if err := s.EnsureValidLandscapeConfig(context.Background()); err != nil {
+		log.Warningf(context.Background(), "Could not ensure valid Landscape configuration: %v", err)
+	}
+
+	return s
 }
 
 // Info returns the current information about the system relevant to the GRPC
@@ -177,16 +184,22 @@ func (s *System) UserProfileDir(ctx context.Context) (wslPath string, err error)
 		return wslPath, err
 	}
 
-	cmd := s.backend.CmdExe(ctx, cmdExe, "/C", "echo %UserProfile%")
+	// Using the 'echo.' syntax instead of 'echo ' because if %USERPROFILE% was set to empty string it would cause the output to be 'ECHO is on'.
+	// With 'echo.%UserProfile%' it correctly prints empty line in that case.
+	cmd := s.backend.CmdExe(ctx, cmdExe, "/C", "echo.%UserProfile%")
 	winHome, err := runCommand(cmd)
 	if err != nil {
 		return wslPath, err
 	}
 
+	trimmed := strings.TrimSpace(string(winHome))
+	if len(trimmed) == 0 {
+		return wslPath, errors.New("%UserProfile% value is empty")
+	}
 	// We have the path from Windows' perspective ( C:\Users\... )
 	// It must be converted to linux ( /mnt/c/Users/... )
 
-	cmd = s.backend.WslpathExecutable(ctx, "-ua", string(winHome))
+	cmd = s.backend.WslpathExecutable(ctx, "-ua", trimmed)
 	winHomeLinux, err := runCommand(cmd)
 	if err != nil {
 		return wslPath, err
@@ -213,6 +226,7 @@ func runCommand(cmd *exec.Cmd) ([]byte, error) {
 
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
+	cmd.Env = append(cmd.Env, "LC_ALL=C") // Ensure that the output is in English
 
 	err := cmd.Run()
 	out := bytes.TrimSpace(stdout.Bytes())
