@@ -22,6 +22,41 @@ const (
 	mockBadOutput                      // A mock that returns a bad value with no error
 )
 
+func TestNew(t *testing.T) {
+	t.Parallel()
+
+	testCases := map[string]struct {
+		systemLandscapeConfigFile string
+	}{
+		"Return a new system": {},
+
+		"Ignore errors when the Landscape config validation failed (only warnings)": {systemLandscapeConfigFile: "invalid_ini.conf"},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			t.Parallel()
+
+			mock := &testutils.SystemMock{
+				FsRoot: testutils.MockFilesystemRoot(t),
+			}
+
+			if tc.systemLandscapeConfigFile != "" {
+				config, err := os.ReadFile(filepath.Join("testdata", "landscape.conf.d", tc.systemLandscapeConfigFile))
+				require.NoError(t, err, "Setup: could not load fixture")
+				err = os.MkdirAll(filepath.Dir(mock.Path(system.LandscapeConfigPath)), 0700)
+				require.NoError(t, err, "Setup: could not create Landscape config dir")
+				err = os.WriteFile(mock.Path(system.LandscapeConfigPath), config, 0600)
+				require.NoError(t, err, "Setup: could not write Landscape system config file")
+			}
+
+			s := system.New(system.WithTestBackend(mock))
+
+			require.NotNil(t, s, "New should return a system object")
+		})
+	}
+}
+
 func TestInfo(t *testing.T) {
 	t.Parallel()
 
@@ -164,12 +199,13 @@ func TestUserProfileDir(t *testing.T) {
 	t.Parallel()
 
 	testCases := map[string]struct {
-		cachedCmdExe      bool
-		cmdExeNotExist    bool
-		cmdExeErr         bool
-		wslpathErr        bool
-		wslpathBadOutput  bool
-		overrideProcMount bool
+		cachedCmdExe           bool
+		cmdExeNotExist         bool
+		cmdExeErr              bool
+		emptyUserprofileEnvVar bool
+		wslpathErr             bool
+		wslpathBadOutput       bool
+		overrideProcMount      bool
 
 		wantErr bool
 	}{
@@ -183,6 +219,7 @@ func TestUserProfileDir(t *testing.T) {
 		"Error finding cmd.exe because there is no Windows FS in /proc/mounts": {wantErr: true, overrideProcMount: true},
 		"Error when cmd.exe does not exist":                                    {cmdExeNotExist: true, overrideProcMount: true, wantErr: true},
 		"Error on cmd.exe error":                                               {cmdExeErr: true, wantErr: true},
+		"Error when UserProfile env var is empty":                              {emptyUserprofileEnvVar: true, wantErr: true},
 		"Error on wslpath error":                                               {wslpathErr: true, wantErr: true},
 		"Error when wslpath returns a bad path":                                {wslpathBadOutput: true, wantErr: true},
 	}
@@ -195,6 +232,9 @@ func TestUserProfileDir(t *testing.T) {
 			if tc.cmdExeErr {
 				mock.SetControlArg(testutils.CmdExeErr)
 			}
+			if tc.emptyUserprofileEnvVar {
+				mock.SetControlArg(testutils.EmptyUserprofileEnvVar)
+			}
 			if tc.wslpathErr {
 				mock.SetControlArg(testutils.WslpathErr)
 			}
@@ -216,7 +256,7 @@ func TestUserProfileDir(t *testing.T) {
 
 			got, err := system.UserProfileDir(context.Background())
 			if tc.wantErr {
-				require.Error(t, err, "Expected UserProfile to return an error")
+				require.Error(t, err, "Expected UserProfile to return an error, but returned %s intead", got)
 				return
 			}
 			require.NoError(t, err, "Expected UserProfile to return no errors")
@@ -390,23 +430,35 @@ func TestLandscapeEnable(t *testing.T) {
 	t.Parallel()
 
 	testCases := map[string]struct {
-		breakWriteConfig     bool
-		breakLandscapeConfig bool
-		breakWSLPath         bool
-		noLandscapeGroup     bool
+		landscapeConfigFile string
+
+		breakWriteConfigDir     bool
+		breakWriteConfig        bool
+		breakLandscapeConfigCmd bool
+		breakWSLPath            bool
+		breakWSLDistroName      bool
+		noLandscapeGroup        bool
 
 		wantErr bool
 	}{
-		"Success":                                                     {},
-		"Success overriding computer_title":                           {},
-		"Success overriding the SSL certficate path":                  {},
-		"Do not append wsl tag when config tag is provided":           {},
-		"Do not append wsl tag when config tag is provided but empty": {},
+		"Appends any required fields - no ssl":                      {landscapeConfigFile: "minimal.conf"},
+		"Transform Windows SSL certificate path":                    {landscapeConfigFile: "windows_ssl_only.conf"},
+		"Transform Windows SSL certificate path with forward slash": {landscapeConfigFile: "windows_ssl_only_forward_slash.conf"},
+		"Refresh computer_title if changed":                         {landscapeConfigFile: "old_computer_title.conf"},
 
-		"Error when the file cannot be parsed":                   {wantErr: true},
-		"Error when the config file cannot be written":           {breakWriteConfig: true, wantErr: true},
-		"Error when the landscape-config command fails":          {breakLandscapeConfig: true, wantErr: true},
+		"Regular with additional keys":            {landscapeConfigFile: "regular.conf"},
+		"Do not modify other sections and keys":   {landscapeConfigFile: "regular_with_extra_keys.conf"},
+		"Reformat Landscape config to proper ini": {landscapeConfigFile: "regular_with_weird_format.conf"},
+
+		"Rerun landscape even without modifications": {landscapeConfigFile: "no_change_needed.conf"},
+
+		"Error when the new config cannot be parsed":             {landscapeConfigFile: "invalid_ini.conf", wantErr: true},
+		"Error when the new config do not have client section":   {landscapeConfigFile: "no_client_section.conf", wantErr: true},
+		"Error when the config directory cannot be created":      {breakWriteConfigDir: true, wantErr: true},
+		"Error when the config file cannot be renamed":           {breakWriteConfig: true, wantErr: true},
+		"Error when the landscape-config command fails":          {breakLandscapeConfigCmd: true, wantErr: true},
 		"Error when failing to override the SSL certficate path": {breakWSLPath: true, wantErr: true},
+		"Error when the can not get WSL Distro name":             {breakWSLDistroName: true, wantErr: true},
 		"Error when the Landscape user does not exist":           {noLandscapeGroup: true, wantErr: true},
 	}
 
@@ -417,12 +469,22 @@ func TestLandscapeEnable(t *testing.T) {
 			ctx := context.Background()
 			s, mock := testutils.MockSystem(t)
 
+			if tc.landscapeConfigFile == "" {
+				tc.landscapeConfigFile = "regular.conf"
+			}
+
+			if tc.breakWriteConfigDir {
+				path := filepath.Dir(mock.Path(system.LandscapeConfigPath))
+				require.NoError(t, os.RemoveAll(path), "Setup: could not remove config directory")
+				require.NoError(t, os.WriteFile(path, nil, 0600), "Setup: could not create file to interfere with config directory creation")
+			}
+
 			if tc.breakWriteConfig {
 				path := mock.Path(system.LandscapeConfigPath)
 				commontestutils.ReplaceFileWithDir(t, path, "Setup: could not create directory to interfere with config file creation")
 			}
 
-			if tc.breakLandscapeConfig {
+			if tc.breakLandscapeConfigCmd {
 				mock.SetControlArg(testutils.LandscapeEnableErr)
 			}
 
@@ -430,28 +492,36 @@ func TestLandscapeEnable(t *testing.T) {
 				mock.SetControlArg(testutils.WslpathErr)
 			}
 
+			if tc.breakWSLDistroName {
+				mock.SetControlArg(testutils.WslpathErr)
+				mock.WslDistroNameEnvEnabled = false
+			}
+
 			if tc.noLandscapeGroup {
 				mock.LandscapeGroupGID = ""
 			}
 
-			config, err := os.ReadFile(filepath.Join(commontestutils.TestFixturePath(t), "landscape.conf"))
+			config, err := os.ReadFile(filepath.Join("testdata", "landscape.conf.d", tc.landscapeConfigFile))
 			require.NoError(t, err, "Setup: could not load fixture")
 
-			err = s.LandscapeEnable(ctx, string(config), "landscapeUID1234")
+			err = s.LandscapeEnable(ctx, string(config))
 			if tc.wantErr {
 				require.Error(t, err, "LandscapeEnable should have returned an error")
 				return
 			}
 			require.NoError(t, err, "LandscapeEnable should have succeeded")
 
+			// landscape --config has been executed
 			exeProof := s.Path("/.landscape-enabled")
 			require.FileExists(t, exeProof, "Landscape executable never ran")
-			out, err := os.ReadFile(exeProof)
-			require.NoErrorf(t, err, "could not read file %q", exeProof)
+
+			// Landscape config file has been written
+			configFileContent, err := os.ReadFile(s.Path(system.LandscapeConfigPath))
+			require.NoErrorf(t, err, "could not read config file %q", s.Path(system.LandscapeConfigPath))
 
 			// We mock the filesystem, and the mocked filesystem root is not the same between
 			// runs, so the golden file would never match. This is the solution:
-			got := strings.ReplaceAll(string(out), mock.FsRoot, "${FILESYSTEM_ROOT}")
+			got := strings.ReplaceAll(string(configFileContent), mock.FsRoot, "${FILESYSTEM_ROOT}")
 
 			want := commontestutils.LoadWithUpdateFromGolden(t, got)
 			require.Equal(t, want, got, "Landscape executable did not receive the right config")
@@ -468,7 +538,6 @@ func TestWindowsHostAddress(t *testing.T) {
 		fileNotExist
 		fileBroken
 		fileIPbroken
-		fileIPisLoopback
 	)
 
 	// copyFile is a helper that copies the appropriate version of a fixture to the desired destination.
@@ -487,8 +556,6 @@ func TestWindowsHostAddress(t *testing.T) {
 			suffix = ".bad"
 		case fileIPbroken:
 			suffix = ".bad-ip"
-		case fileIPisLoopback:
-			suffix = ".loopback"
 		}
 
 		from = from + suffix
@@ -501,36 +568,28 @@ func TestWindowsHostAddress(t *testing.T) {
 	// These are the addresses hard-coded on the fixtures labelled as "good"
 	const (
 		localhost   = "127.0.0.1"
-		nameserver  = "172.22.16.1"
-		degaultGway = "172.25.32.1"
+		defaultGway = "172.25.32.1"
 	)
 
 	testCases := map[string]struct {
 		networkNotNAT bool
 		breakWslInfo  bool
 
-		etcResolv    fileState
 		procNetRoute fileState
 
 		want    string
 		wantErr bool
 	}{
-		"Success without NAT":                          {networkNotNAT: true, want: localhost},
-		"Success with NAT, nameserver is not loopback": {want: nameserver},
-		"Success with NAT, nameserver is loopback":     {etcResolv: fileIPisLoopback, want: degaultGway},
+		"Without NAT": {networkNotNAT: true, want: localhost},
+		"With NAT":    {want: defaultGway},
 
 		// WSL info errors
 		"Error when wslinfo returns an error": {breakWslInfo: true, wantErr: true},
 
-		// NAT errors with broken /etc/resolv.conf
-		"Error with NAT when /etc/resolv.conf does not exist":       {etcResolv: fileNotExist, wantErr: true},
-		"Error with NAT when /etc/resolv.conf is ill-formed":        {etcResolv: fileBroken, wantErr: true},
-		"Error with NAT when /etc/resolv.conf has an ill-formed IP": {etcResolv: fileIPbroken, wantErr: true},
-
 		// NAT errors with loopback nameserver and broken /proc/net/route
-		"Error with NAT when /proc/net/route does not exist":       {etcResolv: fileIPisLoopback, procNetRoute: fileNotExist, wantErr: true},
-		"Error with NAT when /proc/net/route is ill-formed":        {etcResolv: fileIPisLoopback, procNetRoute: fileBroken, wantErr: true},
-		"Error with NAT when /proc/net/route has an ill-formed IP": {etcResolv: fileIPisLoopback, procNetRoute: fileIPbroken, wantErr: true},
+		"Error with NAT when /proc/net/route does not exist":       {procNetRoute: fileNotExist, wantErr: true},
+		"Error with NAT when /proc/net/route is ill-formed":        {procNetRoute: fileBroken, wantErr: true},
+		"Error with NAT when /proc/net/route has an ill-formed IP": {procNetRoute: fileIPbroken, wantErr: true},
 	}
 
 	for name, tc := range testCases {
@@ -547,7 +606,6 @@ func TestWindowsHostAddress(t *testing.T) {
 				mock.SetControlArg(testutils.WslInfoIsNAT)
 			}
 
-			copyFile(t, tc.etcResolv, filepath.Join(commontestutils.TestFamilyPath(t), "etc-resolv.conf"), mock.Path("/etc/resolv.conf"))
 			copyFile(t, tc.procNetRoute, filepath.Join(commontestutils.TestFamilyPath(t), "proc-net-route"), mock.Path("/proc/net/route"))
 
 			got, err := sys.WindowsHostAddress(ctx)
@@ -597,6 +655,121 @@ func TestLandscapeDisable(t *testing.T) {
 	}
 }
 
+func TestEnsureValidLandscapeConfig(t *testing.T) {
+	t.Parallel()
+
+	testCases := map[string]struct {
+		systemLandscapeConfigFile string
+
+		breakWriteConfig        bool
+		breakLandscapeConfigCmd bool
+		breakWSLPath            bool
+		breakWSLDistroName      bool
+		noLandscapeGroup        bool
+
+		wantNoLandscapeConfig    bool
+		wantNoLandscapeConfigCmd bool
+		wantErr                  bool
+	}{
+		"Appends any required fields - no ssl":                      {systemLandscapeConfigFile: "minimal.conf"},
+		"Transform Windows SSL certificate path":                    {systemLandscapeConfigFile: "windows_ssl_only.conf"},
+		"Transform Windows SSL certificate path with forward slash": {systemLandscapeConfigFile: "windows_ssl_only_forward_slash.conf"},
+		"Do not transform Windows SSL certificate empty path":       {systemLandscapeConfigFile: "windows_ssl_empty.conf", wantNoLandscapeConfigCmd: true},
+		"Refresh computer_title if changed":                         {systemLandscapeConfigFile: "old_computer_title.conf"},
+
+		"Regular with additional keys":            {systemLandscapeConfigFile: "regular.conf"},
+		"Do not modify other sections and keys":   {systemLandscapeConfigFile: "regular_with_extra_keys.conf"},
+		"Reformat Landscape config to proper ini": {systemLandscapeConfigFile: "regular_with_weird_format.conf"},
+
+		"Do not rerun landscape without modifications":                             {systemLandscapeConfigFile: "no_change_needed.conf", wantNoLandscapeConfigCmd: true},
+		"No Landscape configuration means no landscape command nor config created": {systemLandscapeConfigFile: "-", wantNoLandscapeConfigCmd: true, wantNoLandscapeConfig: true},
+
+		"Error when the config file cannot be read":              {breakWriteConfig: true, wantErr: true},
+		"Error when the new config cannot be parsed":             {systemLandscapeConfigFile: "invalid_ini.conf", wantErr: true},
+		"Error when the new config do not have client section":   {systemLandscapeConfigFile: "no_client_section.conf", wantErr: true},
+		"Error when the landscape-config command fails":          {breakLandscapeConfigCmd: true, wantErr: true},
+		"Error when failing to override the SSL certficate path": {breakWSLPath: true, wantErr: true},
+		"Error when the can not get WSL Distro name":             {breakWSLDistroName: true, wantErr: true},
+		"Error when the landscape user does not exist":           {noLandscapeGroup: true, wantErr: true},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			t.Parallel()
+
+			ctx := context.Background()
+			s, mock := testutils.MockSystem(t)
+
+			if tc.systemLandscapeConfigFile == "" {
+				tc.systemLandscapeConfigFile = "regular.conf"
+			}
+
+			if tc.breakLandscapeConfigCmd {
+				mock.SetControlArg(testutils.LandscapeEnableErr)
+			}
+
+			if tc.breakWSLPath {
+				mock.SetControlArg(testutils.WslpathErr)
+			}
+
+			if tc.breakWSLDistroName {
+				mock.SetControlArg(testutils.WslpathErr)
+				mock.WslDistroNameEnvEnabled = false
+			}
+
+			if tc.noLandscapeGroup {
+				mock.LandscapeGroupGID = ""
+			}
+
+			if tc.systemLandscapeConfigFile != "-" {
+				if tc.breakWriteConfig {
+					path := mock.Path(system.LandscapeConfigPath)
+					commontestutils.ReplaceFileWithDir(t, path, "Setup: could not create directory to interfere with config file creation")
+				} else {
+					config, err := os.ReadFile(filepath.Join("testdata", "landscape.conf.d", tc.systemLandscapeConfigFile))
+					require.NoError(t, err, "Setup: could not load fixture")
+					err = os.MkdirAll(filepath.Dir(s.Path(system.LandscapeConfigPath)), 0700)
+					require.NoError(t, err, "Setup: could not create Landscape config dir")
+					err = os.WriteFile(s.Path(system.LandscapeConfigPath), config, 0600)
+					require.NoError(t, err, "Setup: could not write Landscape system config file")
+				}
+			}
+
+			err := s.EnsureValidLandscapeConfig(ctx)
+			if tc.wantErr {
+				require.Error(t, err, "EnsureValidLandscapeConfig should have returned an error")
+				return
+			}
+			require.NoError(t, err, "EnsureValidLandscapeConfig should have succeeded")
+
+			// Landscape --config has been executed
+			exeProof := s.Path("/.landscape-enabled")
+			if tc.wantNoLandscapeConfigCmd {
+				require.NoFileExists(t, exeProof, "Landscape executable should not be ran")
+			} else {
+				require.FileExists(t, exeProof, "Landscape executable never ran")
+			}
+
+			// No Landscape config should be on the system.
+			if tc.wantNoLandscapeConfig {
+				require.NoFileExists(t, s.Path(system.LandscapeConfigPath), "Landscape system config not on disk")
+				return
+			}
+
+			// Landscape config file has been kept or modified
+			configFileContent, err := os.ReadFile(s.Path(system.LandscapeConfigPath))
+			require.NoErrorf(t, err, "could not read config file %q", s.Path(system.LandscapeConfigPath))
+
+			// We mock the filesystem, and the mocked filesystem root is not the same between
+			// runs, so the golden file would never match. This is the solution:
+			got := strings.ReplaceAll(string(configFileContent), mock.FsRoot, "${FILESYSTEM_ROOT}")
+
+			want := commontestutils.LoadWithUpdateFromGolden(t, got)
+			require.Equal(t, want, got, "Landscape executable did not receive the right config")
+		})
+	}
+}
+
 func TestRealBackend(t *testing.T) {
 	t.Parallel()
 
