@@ -50,7 +50,7 @@ var (
 
 	// windowsUserProfileDir is the default path used in tests to store the Windows agent address file.
 	windowsUserProfileDir = `D:\Users\TestUser\`
-	linuxUserProfileDir   = "/mnt/d/Users/TestUser/"
+	linuxUserProfileDir   = filepath.Join(defaultWindowsMount, "Users/TestUser/")
 
 	// defaultPublicDir is the default path used in tests to store the address of the Windows Agent service.
 	defaultPublicDir = filepath.Join(linuxUserProfileDir, common.UserProfileDir)
@@ -58,9 +58,6 @@ var (
 	//go:embed filesystem_defaults/os-release
 	defaultOsReleaseContents []byte
 
-	//go:embed filesystem_defaults/resolv.conf
-	defaultResolvConfContents []byte
-
 	//go:embed filesystem_defaults/proc.mounts
 	defaultProcMountsContents []byte
 
@@ -89,8 +86,9 @@ const (
 	LandscapeEnableErr  = "UP4W_LANDSCAPE_ENABLE_ERR"
 	LandscapeDisableErr = "UP4W_LANDSCAPE_DISABLE_ERR"
 
-	WslpathErr       = "UP4W_WSLPATH_ERR"
-	WslpathBadOutput = "UP4W_WSLPATH_BAD_OUTPUT"
+	WslpathErr             = "UP4W_WSLPATH_ERR"
+	WslpathBadOutput       = "UP4W_WSLPATH_BAD_OUTPUT"
+	EmptyUserprofileEnvVar = "UP4W_EMPTY_USERPROFILE_ENV_VAR"
 
 	CmdExeErr = "UP4W_CMDEXE_ERR"
 
@@ -123,7 +121,7 @@ func MockSystem(t *testing.T) (*system.System, *SystemMock) {
 
 	distroHostname := "TEST_DISTRO_HOSTNAME"
 	mock := &SystemMock{
-		FsRoot:                  mockFilesystemRoot(t),
+		FsRoot:                  MockFilesystemRoot(t),
 		WslDistroName:           "TEST_DISTRO",
 		DistroHostname:          &distroHostname,
 		WslDistroNameEnvEnabled: true,
@@ -146,6 +144,12 @@ func (m *SystemMock) SetControlArg(arg controlArg) {
 
 // Path prepends FsRoot to a path.
 func (m *SystemMock) Path(path ...string) string {
+	// We need to special case our local certificate to not prepend m.FsRoot to it and modify our idempotent path
+	for _, p := range path {
+		if strings.Contains(p, "idempotent") {
+			return filepath.Join(path...)
+		}
+	}
 	path = append([]string{m.FsRoot}, path...)
 	return filepath.Join(path...)
 }
@@ -508,10 +512,18 @@ func WslPathMock(t *testing.T) {
 			if envExists(WslpathErr) {
 				return exitError
 			}
+			// That's what wslpath returns when it's called with -ua followed by an empty string.
+			cwd, err := os.Getwd()
+			if err != nil {
+				fmt.Fprintf(os.Stderr, "Could not get current working directory: %v", err)
+			}
 
 			stdout, ok := map[string]string{
-				windowsUserProfileDir:           linuxUserProfileDir,
-				`D:\Users\TestUser\certificate`: filepath.Join(defaultWindowsMount, "Users/TestUser/certificate"),
+				windowsUserProfileDir:                   linuxUserProfileDir,
+				`D:\Users\TestUser\certificate`:         filepath.Join(defaultWindowsMount, "Users/TestUser/certificate"),
+				"D:/Users/TestUser/certificate":         filepath.Join(defaultWindowsMount, "Users/TestUser/certificate"),
+				"/idempotent/path/to/linux/certificate": "/idempotent/path/to/linux/certificate",
+				"":                                      cwd,
 			}[argv[1]]
 
 			if !ok {
@@ -597,7 +609,7 @@ func CmdExeMock(t *testing.T) {
 			return exitBadUsage
 		}
 
-		if argv[1] != "echo %UserProfile%" {
+		if argv[1] != "echo.%UserProfile%" {
 			fmt.Fprintf(os.Stderr, "Mock not implemented for args %q\n", argv)
 			return exitBadUsage
 		}
@@ -605,6 +617,10 @@ func CmdExeMock(t *testing.T) {
 		if envExists(CmdExeErr) {
 			return exitError
 		}
+		if envExists(EmptyUserprofileEnvVar) {
+			fmt.Print("\r\n")
+			return exitOk
+		}
 
 		fmt.Fprintln(os.Stdout, windowsUserProfileDir)
 		return exitOk
@@ -644,9 +660,9 @@ func mockMain(t *testing.T, f func(argv []string) exitCode) {
 	syscall.Exit(exit)
 }
 
-// mockFilesystemRoot sets up a skelleton filesystem with files used by the wsl-pro-service and returns
+// MockFilesystemRoot sets up a skelleton filesystem with files used by the wsl-pro-service and returns
 // its root dir.
-func mockFilesystemRoot(t *testing.T) (rootDir string) {
+func MockFilesystemRoot(t *testing.T) (rootDir string) {
 	t.Helper()
 
 	rootDir = t.TempDir()
@@ -655,9 +671,6 @@ func mockFilesystemRoot(t *testing.T) (rootDir string) {
 	err := os.MkdirAll(filepath.Join(rootDir, "etc"), 0750)
 	require.NoError(t, err, "Setup: could not create mock /etc/")
 
-	err = os.WriteFile(filepath.Join(rootDir, "etc/resolv.conf"), defaultResolvConfContents, 0600)
-	require.NoError(t, err, "Setup: could not write mock /etc/resolv.conf")
-
 	err = os.WriteFile(filepath.Join(rootDir, "etc/os-release"), defaultOsReleaseContents, 0600)
 	require.NoError(t, err, "Setup: could not write mock /etc/os-release")
 
